<?php

namespace Classes\Helper;

if (!defined('_PS_VERSION_')) {
    exit;
}

/**
 * Class to establish Logger in order for information to be logged to zipMoney-Payments.log
 *
 * @class    Logger
 * @version  1.0.2
 * @package  zipMoney Payments
 * @author   zipMoney Payments
 */
class Logger
{
    private static $fileLogger;  // Log file name
    private static $fileName = "zipMoney-Payments.log";  // Log file name

    const ERROR   = 7;  // Error: error conditions
    const WARN    = 8;  // Warning: warning conditions
    const INFO    = 9;  // Informational: informational messages
    const DEBUG   = 10;  // Debug: debug messages

    /**
     * Sets up the logger and defines the log file path.
     *
     * @access public
     */
    public static function setup()
    {
        self::$fileLogger = new \FileLogger;

        if (version_compare(_PS_VERSION_, '1.7', '>=')) {
            self::$fileLogger->setFilename(_PS_ROOT_DIR_ . '/var/logs/' . self::$fileName);
        } else {
            self::$fileLogger->setFilename(_PS_ROOT_DIR_ . '/log/' . self::$fileName);
        }
    }

    /**
     * Logs the debug message
     *
     * @param string $message, int $level
     * @access public
     */
    public static function writeLog($message, $level = self::INFO, $overideConfig = false)
    {
        if (!$message) {
            return false;
        }

        $configLevel = (\Tools::getValue('zm_logging', \Configuration::get('ZM_LOGGING')));

        if (!$overideConfig) { // You can use this variable to log regardless of config settings.
            if (!$configLevel) {
                return false;
            }
        }

        // Errors are always logged.
        if ($configLevel < 7) {
            $configLevel = self::INFO; // default log level
        }

        if ($level > $configLevel) {
            return false;
        }

        switch ($level) {
            case self::DEBUG:
                self::$fileLogger->level = \FileLogger::DEBUG;
                self::$fileLogger->logDebug($message);
                break;

            case self::INFO:
                self::$fileLogger->level = \FileLogger::INFO;
                self::$fileLogger->logInfo($message);
                break;

            case self::WARN:
                self::$fileLogger->level = \FileLogger::WARNING;
                self::$fileLogger->logWarning($message);
                break;

            case self::ERROR:
                self::$fileLogger->level = \FileLogger::ERROR;
                self::$fileLogger->logError($message);
                break;

            default:
                self::$fileLogger->level = \FileLogger::INFO;
                self::$fileLogger->logInfo($message);
                break;
        }
    }

    /**
     * Logs the debug message
     *
     * @access public
     */
    public static function debug($message, $overideConfig = false)
    {
        self::writeLog($message, self::DEBUG, $overideConfig);
    }

    /**
     * Logs the info message
     *
     * @access public
     */
    public static function info($message)
    {
        self::writeLog($message, self::INFO);
    }

    /**
     * Logs the warnings
     *
     * @access public
     */
    public static function warn($message)
    {
        self::writeLog($message, self::WARN);
    }

    /**
     * Logs the error message
     *
     * @access public
     */
    public static function error($message)
    {
        self::writeLog($message, self::ERROR);
    }
}
