<?php

namespace Classes\Helper;

use PrestaShop\PrestaShop\Adapter\ServiceLocator;

if (!defined('_PS_VERSION_')) {
    exit;
}

/**
 * Helper Class for that allows primary API to be mockable.
 *
 * @class    Sdk
 * @version  1.0.2
 * @package  zipMoney Payments
 * @author   zipMoney Payments
 */
class Sdk
{

    private static $_checkoutApi;
    private static $_chargeApi;
    private static $_refundApi;
    const CHECKOUT_API_CLASS = "\\zipMoney\\Api\\CheckoutsApi";
    const CHARGE_API_CLASS = "\\zipMoney\\Api\\ChargesApi";
    const REFUND_API_CLASS = "\\zipMoney\\Api\\RefundsApi";

    /**
     * Get dependency api instance.
     *
     * @return Class $api
     */
    public static function getApi($class)
    {
        switch ($class) {
            case self::CHECKOUT_API_CLASS:
                if (!is_null(self::$_checkoutApi)) {
                    return self::$_checkoutApi;
                } else {
                    return self::setApi(self::CHECKOUT_API_CLASS);
                }
                break;
            case self::CHARGE_API_CLASS:
                if (!is_null(self::$_chargeApi)) {
                    return self::$_chargeApi;
                } else {
                    return self::setApi(self::CHARGE_API_CLASS);
                }
                break;
            case self::REFUND_API_CLASS:
                if (!is_null(self::$_refundApi)) {
                    return self::$_refundApi;
                } else {
                    return self::setApi(self::REFUND_API_CLASS);
                }
                break;
            default:
                break;
        }
    }

    /**
     * Set dependency api instance.
     *
     * @return Class $api
     */
    public static function setApi($class = "", $api = null)
    {
        if (!is_null($api)) {
            switch ($class) {
                case self::CHECKOUT_API_CLASS:
                    self::$_checkoutApi = $api;
                    break;
                case self::CHARGE_API_CLASS:
                    self::$_chargeApi = $api;
                    break;
                case self::REFUND_API_CLASS:
                    self::$_refundApi = $api;
                    break;
                default:
                    break;
            }
        } else {
            switch ($class) {
                case self::CHECKOUT_API_CLASS:
                    self::$_checkoutApi = new $class;
                    $api = self::$_checkoutApi;
                    break;
                case self::CHARGE_API_CLASS:
                    self::$_chargeApi = new $class;
                    $api = self::$_chargeApi;
                    break;
                case self::REFUND_API_CLASS:
                    self::$_refundApi = new $class;
                    $api = self::$_refundApi;
                    break;
                default:
                    break;
            }
        }
        return $api;
    }
}
