<?php
if (!defined('_PS_VERSION_')) {
    exit;
}

use Classes\zipMoney;
use Classes\zipMoneyStandard;
use Classes\Helper\Api as ApiHelper;
use Classes\Helper\Logger as zipMoneyLogger;
use zipMoney\ApiException;

/**
 * Controller class for payment route
 *
 * @class    ZipmoneyPaymentPaymentModuleFrontController
 * @version  1.0.2
 * @package  zipMoney Payments
 * @author   zipMoney Payments
 */
class ZipmoneyPaymentPaymentModuleFrontController extends ModuleFrontController
{

    private $_cart;

    /**
     * Initialises content
     */
    public function initContent()
    {
        parent::initContent();

        $redirect_url = $this->paymentInit();

        if ($redirect_url) {
            die(json_encode(array('redirect_uri' => $redirect_url, 'success' => true)));
        } else {
            //Handle error
            $redirect_url = $this->context->link->getPageLink('order');
            die(json_encode(array('redirect_uri' => $redirect_url, 'message' => $this->module->l('An error occurred while getting the redirect url from Zip Payment.', 'ZipmoneyPayment'), 'success' => false)));
        }
    }

    /**
     * Initialises payment
     *
     * @return String redurectUri [The uri to redirect/access in order for the end user to authorise the payment.]
     */
    public function paymentInit()
    {
        $cart = $this->getCart();

        $zip = ApiHelper::getApi();

        if (!$cart) {
            throw new \Exception("Error Processing Request", 1);
        }

        // Don't create an order until customer is approved for their purchase. Therefore the $cart is all that can be used.
        $redirectUri = $zip->handleCheckout($cart);
        zipMoneyLogger::info("Redirect Uri := " . $redirectUri);

        return $redirectUri;
    }

    /**
     * Returns the cart if it is set, else sets it.
     *
     * @return Class cart
     */
    public function getCart()
    {

        if (!is_null($this->_cart)) {
            return $this->_cart;
        } else {
            return $this->setCart();
        }
    }

    /**
     * Sets the cart to provided cart, otherwise sets cart to session cart.
     * This makes the cart mockable.
     * @param Class $cart
     * @return Class $cart
     */
    public function setCart($cart = null)
    {
        if (!is_null($cart)) {
            $this->_cart = $cart;
        } else {
            $this->_cart = $this->context->cart;
        }
        return $this->_cart;
    }
}
