<?php

/**
 * Controller class for payment route
 *
 * @class    ZipmoneyPaymentValidationModuleFrontController
 * @version  1.0.2
 * @package  zipMoney Payments
 * @author   zipMoney Payments
 */

use Classes\zipMoney;
use Classes\zipMoneyStandard;
use Classes\Helper\Api as ApiHelper;
use Classes\Helper\Sdk;
use Classes\Helper\Logger as zipMoneyLogger;
use zipMoney\ApiException;
use zipMoney\Api\ChargesApi as Charge;

/**
 * Controller class for validation route
 *
 * @class    ZipmoneyPaymentValidationModuleFrontController
 * @version  1.0.2
 * @package  zipMoney Payments
 * @author   zipMoney Payments
 */
class ZipmoneyPaymentValidationModuleFrontController extends ModuleFrontController
{

    private $_cart;

    /**
     * Entry point for the validation controller
     *
     * @access public
     */
    public function postProcess()
    {
        try {
            $zip = ApiHelper::getApi();
            \zipMoney\Configuration::getDefaultConfiguration()->setApiKey('Authorization', "Bearer " . $zip->getMerchantKey());
            \zipMoney\Configuration::getDefaultConfiguration()->setEnvironment($zip->getEnvironment());
            \zipMoney\Configuration::getDefaultConfiguration()->setPlatform($zip->getPlatform());

            $cart = $this->getCart();

            $checkout_id = (Tools::getValue('checkoutId', 0));
            $resultFromQuery = (Tools::getValue('result', 0));
            //$is_iframe = (Tools::getValue('iframe', 0)); this iframe check is disable until we fix the checkout js iframe issue
            $is_iframe = false;
            $result = strtolower($resultFromQuery);
            if ($is_iframe) {
                $this->context->smarty->assign('checkoutId', $checkout_id);
                $this->context->smarty->assign('state', $result);
                $params = [
                    'result' => $result,
                    'checkoutId' => $checkout_id,
                ];
                $redirectUri = $this->context->link->getModuleLink('zipmoneypayment', 'validation', $params);
                $this->context->smarty->assign('redirectUrl', $redirectUri);
                $this->setTemplate('module:zipmoneypayment/views/templates/hooks/zipmoney_iframe.tpl');
            } else {
                if (!$checkout_id || !$result) {
                    throw new \Exception("Checkout Id or Result not set.");
                }

                zipMoneyLogger::info("Accessing the validation URL: " . "http://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]" . " with the result: " . $result);

                switch ($result) {
                    case 'approved':
                        $redirect_url = $this->handleApproval($checkout_id);

                        if ($redirect_url) {
                            \Tools::redirect($redirect_url);
                        } else {
                            \Tools::redirect($this->context->link->getModuleLink('zipmoneypayment', 'zipmoney_error', array()));
                        }

                        break;
                    case 'declined':
                        if (version_compare(_PS_VERSION_, '1.7', '<')) {
                            $this->setTemplate('1.5/decline.tpl');
                        } else {
                            $this->setTemplate('module:zipmoneypayment/views/templates/front/decline.tpl');
                        }
                        break;
                    case 'cancelled':
                        \Tools::redirect($this->context->link->getPageLink('order', true, null, 'step=3'));
                        break;
                    case 'referred':
                        $noSessionCart = $this->removeCart();
                        if ($noSessionCart) {
                            $cart_id = $noSessionCart->id;
                        }
                        if (version_compare(_PS_VERSION_, '1.7', '<')) {
                            $this->setTemplate('1.5/refer.tpl');
                        } else {
                            $this->setTemplate('module:zipmoneypayment/views/templates/front/refer.tpl');
                        }
                        break;
                    default:
                        \Tools::redirect($this->context->link->getPageLink('index.php'));
                        break;
                }
            }
        } catch (ApiException $e) {
            $error = \Tools::jsonEncode(array('message' => $e->getResponseBody()));
            zipMoneyLogger::error("Error:-" . $e->getCode() . "-" . json_encode($e->getResponseBody()));
            \Tools::redirect($this->context->link->getModuleLink('zipmoneypayment', 'zipmoney_error', array()));
        } catch (Exception $e) {
            $error = \Tools::jsonEncode(array('message' => $e->getMessage()));
            zipMoneyLogger::error("Payment Exception: " . $error);
            \Tools::redirect($this->context->link->getModuleLink('zipmoneypayment', 'zipmoney_error', array()));
        }
    }

    /**
     * Function called when checkout is approved and ready to be charged.
     *
     * If the user is referred then store the checkout against the cart for the user is approved and the checkout is resumed.
     * Validate the information
     * Convert information (cart) to order
     * Handle API interaction
     * Modify order accordingly, depending on the capture status.
     *
     * @access public
     * @param string $checkout_id
     * @return string [A url to redirect to]
     */
    public function handleApproval($checkout_id)
    {

        $zip_pending = zipMoney::getOrderStatus("zip_pending");
        $zip_authorised = zipMoney::getOrderStatus("zip_authorised");

        $zip = ApiHelper::getApi();
        $cart = $this->getCart();
        $stored_id = $this->getCheckoutDetails($checkout_id);
        if (empty($stored_id)) {
            $stored_id = $this->getCheckoutDetails('au-' . $checkout_id);
            if ($stored_id) {
                $checkout_id = 'au-' . $checkout_id;
            }
        }

        if ($stored_id != $cart->id) {
            $cart = new \Cart($stored_id);
            $this->setCart($cart);

            if (_PS_VERSION_ >= '1.5') {
                if ($this->context->cookie->isLogged()) {
                    $this->context->cookie->logout();
                }
            } else {
                if (Customer::isLogged()) {
                    Customer::logout();
                }
            }
        }


        $authorized = false;
        foreach ($zip->getPaymentModules() as $module) {
            if ($module['name'] == 'zipmoneypayment') {
                $authorized = true;
                break;
            }
        }
        $customer = new Customer($cart->id_customer);
        $customerValidated = \Validate::isLoadedObject($customer);
        $itemsAreAvailable = true;
        if ($items = $cart->getProducts()) {
            foreach ($items as $item) {
                $noStock = (int)(\Configuration::get('PS_STOCK_MANAGEMENT')) ? (int)$item['quantity_available'] == 0 : false;
                if ((int)$item['available_for_order'] != 1 || (int)$item['active'] != 1 || $noStock) {
                    $itemsAreAvailable = false;
                }
            }
        }

        if (!$itemsAreAvailable) {
            zipMoneyLogger::warn("Some items are no longer available.");
            // Redirect
            \Tools::redirect($this->context->link->getModuleLink('zipmoneypayment', 'zipmoney_error', array()));
        }
        if (!$customerValidated) {
            zipMoneyLogger::warn("Customer is not validated.");
        }
        if (!$authorized) {
            zipMoneyLogger::warn("zipmoneypayment method is not available/authorised.");
        }

        if (!$cart || !$itemsAreAvailable || !$customerValidated || !$authorized || $cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0 || !$this->module->active) {
            zipMoneyLogger::warn("Cannot proceed with the order and capture.");
            return $this->context->link->getPageLink('order', true, null, 'step=1');
        }

        $zipMoneyPayment = new \ZipmoneyPayment;
        $moduleName = $zipMoneyPayment->name;

        $order = $zip->validateOrder($cart, $zip_pending, 0, $moduleName, null, array(), (int)$cart->id_currency, false, $customer->secure_key);

        zipMoneyLogger::debug("Order created: " . json_encode($order));

        $response = $zip->handleCharge($order, $checkout_id);

        if (array_key_exists('redirect', $response)) {
            Tools::redirect($response['redirect']);
        }

        if (method_exists($response, 'getId')) {
            switch ($response->getState()) {
                case "captured":
                case "authorised":
                case "approved":
                    $order->addOrderPayment((float)$response->getAmount(), $moduleName, $response->getId());
                    $history = new \OrderHistory();
                    $history->id_order = (int)$order->id;
                    $is_capture = \Configuration::get('ZM_CAPTURE_METHOD') == 'capture' ? true : false;
                    if ($is_capture) {
                        $order_state = (int)\Configuration::get('PS_OS_PAYMENT');
                        if ((int)$order->current_state == (int)\Configuration::get('PS_OS_OUTOFSTOCK_UNPAID')) {
                            $order_state = (int)\Configuration::get('PS_OS_OUTOFSTOCK_PAID');
                        }
                        $history->changeIdOrderState($order_state, (int)$order->id, true);
                        zipMoneyLogger::info(sprintf("zipMoney Capture Success [Order Id %s]. Status changed to processing. ", $order->id));
                    } else {
                        $history->changeIdOrderState($zip_authorised, (int)$order->id, true);
                        zipMoneyLogger::info(sprintf("zipMoney Authorise Success [Order Id %s]. Status changed to zip_authorised. ", $order->id));
                    }
                    $history->addWithemail();
                    break;
                default:
                    break;
            }

            return $this->context->link->getPageLink('order-confirmation', true, null, 'id_cart=' . (int)$order->id_cart . '&id_module=' . (int)$this->module->id . '&id_order=' . $order->id . '&key=' . $customer->secure_key);
        }
        return null;
    }

    /**
     * Initialises payment
     *
     * @access public
     * @return Object redurectUri [The uri to redirect/access in order for the end user to authorise the payment.]
     */
    public function getCart()
    {
        if (!is_null($this->_cart)) {
            return $this->_cart;
        } else {
            return $this->setCart();
        }
    }

    /**
     * Returns the cart if it is set, else sets it.
     *
     * @access public
     * @return Class cart
     */
    public function setCart($cart = null)
    {
        if (!is_null($cart)) {
            $this->_cart = $cart;
            if (is_null($this->context->cart)) {
                $this->context->cart = $this->_cart;
            }
        } else {
            $this->_cart = $this->context->cart;
        }
        return $this->_cart;
    }

    /**
     * Saves and resets cart session
     *
     * @access public
     * @return Class cart
     */
    public function removeCart()
    {
        $dup = $this->_cart->duplicate();
        if ($dup['success']) {
            $this->_cart->delete();
            zipMoneyLogger::debug("Duplicated Cart: " . json_encode($dup));

            return $dup['cart'];
        } else {
            zipMoneyLogger::error("Duplicated Cart ERROR: " . json_encode($dup));

            return $this->_cart;
        }
    }

    protected function getCheckoutDetails($checkout_id)
    {
        // Configure API Credentials
        $zip = ApiHelper::getApi();
        \zipMoney\Configuration::getDefaultConfiguration()->setApiKey('Authorization', "Bearer " . $zip->getMerchantKey());
        \zipMoney\Configuration::getDefaultConfiguration()->setEnvironment($zip->getEnvironment());
        \zipMoney\Configuration::getDefaultConfiguration()->setPlatform($zip->getPlatform());
        $checkoutApi = Sdk::getApi(Sdk::CHECKOUT_API_CLASS);
        zipMoneyLogger::error("checkout id : " . $checkout_id);
        $checkout = $checkoutApi->checkoutsGet($checkout_id);
        if (!isset($checkout['order']['reference'])) {
            return false;
        }
        zipMoneyLogger::error("Cart Id: " . $checkout['order']['reference']);
        return $checkout['order']['reference'];
    }
}
