<?php

use Classes\Helper\Sdk;
use Classes\Helper\Api as ApiHelper;

/**
 * @class    ZipmoneyPaymentValidationModuleFrontControllerTest
 * @version  1.0.2
 * @package  zipMoney Payments
 * @author   zipMoney Payments
 */
class ZipmoneyPaymentValidationModuleFrontControllerTest extends ModulePrestaShopPHPUnit
{
    public $module = null;

    /**
     * Test Module setting up validation route
     *
     * @access public
     */
    public function setUp()
    {
        require_once dirname(__FILE__) . "/../../../controllers/front/validation.php";

        $this->module = \Module::getInstanceByName(Tools::strtolower('zipmoneypayment'));

        $_POST['module'] = 'zipmoneypayment';

        $this->context = \Context::getContext();

        $this->controller = \Controller::getController("ZipmoneyPaymentValidationModuleFrontController");
        $this->controller->module = $this->module;
    }

    /**
     * Test Module for handleApproval method in validation route
     *
     * @access public
     */
    public function testHandleApproval()
    {
        $cart = $this->getMockBuilder('Cart')
            ->setConstructorArgs(array(2))
            ->getMock();
        $cart->method('getProducts')
            ->willReturn(array(array(
                "available_for_order" => "1",
                "active" => "1",
                "quantity_available" => "1",
                "id_product" => 3,
                "name" => "the best product ever",
                "price_wt" => 29,
                "quantity" => 1
            )));
        $cart->method('getOrderTotal')
            ->willReturn(55);
        $cart->method('getTotalShippingCost')
            ->willReturn(10);
        $cart->id = 20;
        $cart->id_customer = 1;
        $cart->id_address_delivery = 1;
        $cart->id_address_invoice = 1;

        $order = $this->getMockBuilder('Order')
            ->setConstructorArgs(array(2))
            ->getMock();
        $order->total_paid = 100;
        $order->id_currency = 1;
        $order->id_customer = 1;
        $order->id = 19;
        $order->id_cart = $cart->id;
        $order->method('getOrderPaymentCollection')->willReturn(array('testPaymentId' => (object)array('transaction_id' => 'AAAAAAA')));

        $api = $this->getMockBuilder(ApiHelper::API_CLASS)->setMethods(array('validateOrder', 'getCartIdFromCheckoutId', 'getPaymentModules', 'getCartFromOrder'))->getMock();
        $api->method('validateOrder')->willReturn($order);
        $api->method('getCartIdFromCheckoutId')->willReturn(null);
        $api->method('getPaymentModules')->willReturn(array(array("name" => "zipmoneypayment")));
        $api->method('getCartFromOrder')->willReturn($cart);
        ApiHelper::setApi($api);

        $chargeApi = $this->getMockBuilder(Sdk::CHARGE_API_CLASS)->getMock();
        $chargeOut = new \zipMoney\Model\Charge();
        $chargeOut->setReference("This is a test validation charge output.");
        $chargeOut->setState('authorised');
        $chargeOut->setId("49");
        $chargeApi->method('chargesCreate')->willReturn($chargeOut);

        Sdk::setApi(Sdk::CHARGE_API_CLASS, $chargeApi);

        $this->controller->setCart($cart);

        $this->controller->init();
        $response = $this->controller->handleApproval('CCCCCC');

        // echo "Validation Response Reference: " . $response;
        // echo "getPageLinK: " . $this->context->link->getPageLink('order-confirmation', true, null);

        $isConfirmed = strpos($response, $this->context->link->getPageLink('order-confirmation', true, null)) === 0;
        $this->assertTrue($isConfirmed);
    }
}
