<?php

use Classes\Helper\Api as ApiHelper;
use Classes\Helper\Sdk;

/**
 * @class    ZipmoneyPaymentTest
 * @version  1.0.2
 * @package  zipMoney Payments
 * @author   zipMoney Payments
 */
class ZipmoneyPaymentTest extends ModulePrestaShopPHPUnit
{
    public $module = null;

    /**
     * Test Module to setup the extension
     *
     * @access public
     */
    public function setUp()
    {
        $this->module = Module::getInstanceByName(Tools::strtolower('zipmoneypayment'));
        $this->module->zm_is_active = true;
        $this->module->active = true;
        $this->module->zm_is_sandbox = true;

        $this->api = $this->getMockBuilder(ApiHelper::API_CLASS)
            ->setMethods(array('getEnvironment', 'getMerchantKey'))
            ->getMock();
        $this->api->method('getEnvironment')
            ->willReturn('sandbox');
        $this->api->method('getMerchantKey')
            ->willReturn('test');

        $this->order = $this->getMockBuilder('Order')
            //->setConstructorArgs(array(2))
            ->getMock();

        $this->order->id = 1000000;
        $this->order->id_cart = 1000002;
        $this->order->id_currency = 1;
        $this->order->payment = "zipmoneypayment";
        $this->order->module  = "zipmoneypayment";

        $this->order->method('getOrderPaymentCollection')->willReturn(array('testPaymentId' => (object)array('transaction_id' => 'AAAAAAA')));
    }

    /**
     * Test Module to determine correct module name
     * @access public
     */
    public function testModuleName()
    {
        $this->assertTrue($this->module->name == 'zipmoneypayment');
    }

    /**
     * Test Module for hookPayment method
     *
     * @access public
     */
    public function testHookPayment()
    {
        $cart = new Cart();
        $cart->id_currency = 1;

        if (version_compare(_PS_VERSION_, '1.7', '>=')) {
            $cart = new Cart();
            $cart->id_currency = 1;

            $options = $this->module->hookPaymentOptions(array('cart' => $cart));
            $option = $options[0];
            $this->assertInternalType('array', $options);
            $this->assertTrue($option instanceof \PrestaShop\PrestaShop\Core\Payment\PaymentOption);
            $this->assertEquals($option->getLogo(), $this->module->getMethodIcon());
            $this->assertNotEmpty($option->getCallToActionText());
            $this->assertNotEmpty($option->getAdditionalInformation());
        } else {
            $tpl = $this->module->hookPayment(array('cart' => $cart));
            $this->assertEquals(\Context::getContext()->smarty->getTemplateVars('zm_logo_url'), $this->module->getMethodIcon());
            $this->assertNotEmpty(\Context::getContext()->smarty->getTemplateVars('checkout_title'));
            $this->assertEquals(\Context::getContext()->smarty->getTemplateVars('checkoutUri'), \Context::getContext()->link->getModuleLink($this->module->name, 'payment', array(), true));
            $this->assertEquals(\Context::getContext()->smarty->getTemplateVars('redirectUri'), \Context::getContext()->link->getModuleLink($this->module->name, 'validation', array(), true));
            $this->assertEquals(\Context::getContext()->smarty->getTemplateVars('zm_in_context'), $this->module->useIframeCheckout() ? 'true' : 'false');
            $this->assertEquals($this->module->fetchTemplate('views/templates/hooks/payment.tpl'), $tpl);
        }
    }

    /**
     * Test Module for _fullRefund method
     *
     * @access public
     */
    public function testFullRefund()
    {
        $this->order->method('getProducts')
            ->willReturn(array(array("id_order_detail" => 1, "product_price_wt" => 15, "product_quantity" => 1, "product_quantity_refunded" => 0)));

        $this->order->total_shipping = 5;
        $this->order->total_wrapping = 0;
        $this->order->total_discounts = 0;
        $this->order->total_paid_real = 20;

        $refundApi = $this->getMockBuilder(Sdk::REFUND_API_CLASS)->getMock();
        $refundOut = new \zipMoney\Model\Refund();
        $refundOut->setChargeId("This_is_a_test_id");
        $refundApi->method('refundsCreate')->willReturn($refundOut);

        Sdk::setApi(Sdk::REFUND_API_CLASS, $refundApi);

        $response = $this->invoke($this->module, "_fullRefund", $this->order, true);
        // echo "Refund Response Id: " . $response->getChargeId() . " \n";
        $this->assertEquals($refundOut->getChargeId(), $response->getChargeId());
    }

    public function testHookActionOrderStatusPostUpdate()
    {
        $this->order->total_paid = 100;

        $params = array(
            'newOrderStatus' => (object)array("template" => "preparation"),
            'id_order' => $this->order
        );

        $chargeApi = $this->getMockBuilder(Sdk::CHARGE_API_CLASS)->getMock();
        $chargeOut = new \zipMoney\Model\Charge();
        $chargeOut->setReference("This is a test charge output.");
        $chargeOut->setState('authorised');
        $chargeOut->setId("49");
        $chargeApi->method('chargesCapture')->willReturn($chargeOut);
        $chargeApi->method('chargesCancel')->willReturn($chargeOut);
        $chargeApi->method('chargesRetrieve')->willReturn($chargeOut);

        Sdk::setApi(Sdk::CHARGE_API_CLASS, $chargeApi);

        // Capture an authorised
        // $response = $this->invoke($this->module, "hookActionOrderStatusPostUpdate", $params);
        $response = $this->module->hookActionOrderStatusPostUpdate($params);
        // echo "Capture Response Reference: " . $response->getReference() . "\n";
        $this->assertEquals($response->getReference(), $chargeOut->getReference());

        // Cancel a captured
        $params['newOrderStatus']->template = "order_canceled";
        // $response = $this->invoke($this->module, "hookActionOrderStatusPostUpdate", $params);
        $response = $this->module->hookActionOrderStatusPostUpdate($params);
        // echo "Capture Response Reference: " . $response->getReference() . "\n";
        $this->assertEquals($response->getReference(), $chargeOut->getReference());
    }
}
