<?php

use zipMoney\Model\CurrencyUtil;

if (!defined('_PS_VERSION_')) {
    exit;
}

/**
 * @class    ZipmoneyPayment
 * @version  1.0.5
 * @package  Zip
 * @author   Zip
 */
class ZipmoneyPayment extends PaymentModule
{
    private $_logo_src_url = "https://static.zipmoney.com.au/logo/25px/zip.png";
    private $_html;

    public function __construct()
    {
        $this->name = 'zipmoneypayment';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.5';
        $this->author = 'Zip';
        $this->need_instance = 0;
        $this->bootstrap = true;
        $this->controllers = array('payment', 'webhook');

        parent::__construct();

        $this->displayName = $this->l('Zip');
        $this->description = $this->l('Sell more online & in-store with Zip.
Give your customers the power to pay later, interest free and watch your sales grow.
Take advantage of our fast-growing customer base, proven revenue uplift, fast and simple integration.');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

        $config = Configuration::getMultiple(
            array(
                'ZM_IS_ACTIVE',
                'ZM_MERCHANT_PUBLIC_KEY',
                'ZM_MERCHANT_PRIVATE_KEY',
                'ZM_ENVIRONMENT',
                'ZM_CAPTURE_METHOD',
                'ZM_MINIMUM_ORDER',
                'ZM_LOGGING',
                'ZM_PRODUCT',
                'ZM_CHECKOUT_TEXT'
            )
        );

        if (!empty($config['ZM_IS_ACTIVE'])) {
            $this->zm_is_active = $config['ZM_IS_ACTIVE'];
        }

        if (!empty($config['ZM_MERCHANT_PUBLIC_KEY'])) {
            $this->zm_merchant_public_key = $config['ZM_MERCHANT_PUBLIC_KEY'];
        }

        if (!empty($config['ZM_MERCHANT_PRIVATE_KEY'])) {
            $this->zm_merchant_private_key = $config['ZM_MERCHANT_PRIVATE_KEY'];
        }

        if (!empty($config['ZM_ENVIRONMENT'])) {
            $this->zm_environment = $config['ZM_ENVIRONMENT'];
        }

        if (!empty($config['ZM_CAPTURE_METHOD'])) {
            $this->zm_capture_method = $config['ZM_CAPTURE_METHOD'];
        }

        if (!empty($config['ZM_MINIMUM_ORDER'])) {
            $this->zm_minimum_order = $config['ZM_MINIMUM_ORDER'];
        }

        if (!empty($config['ZM_LOGGING'])) {
            $this->zm_logging = $config['ZM_LOGGING'];
        }

        if (!empty($config['ZM_PRODUCT'])) {
            $this->zm_product = $config['ZM_PRODUCT'];
        }
        if (!empty($config['ZM_CHECKOUT_TEXT'])) {
            $this->zm_checkout_text = $config['ZM_CHECKOUT_TEXT'];
        }

        if (!isset($this->zm_merchant_public_key) || !isset($this->zm_merchant_private_key)) {
            $this->warning = $this->trans('Account owner and account details must be configured before using this module.', array(), 'Modules.Zipmoneypayment.Zipmoneypayment');
        }

        require_once __DIR__ . '/vendor/autoload.php';

        $this->zipWidgets = new \Classes\zipMoneyWidgets($this->context);

        \Classes\zipMoney::bootstrap();
    }

    /**
     * Module installation.
     *
     * @access public
     * @return bool Success of the installation
     */
    public function install()
    {
        if (
            !parent::install() ||
            !$this->registerHook('payment') ||
            !$this->registerHook('paymentReturn') ||
            !$this->registerHook('adminOrder') ||
            !$this->registerHook('actionOrderSlipAdd') ||
            !$this->registerHook('actionOrderStatusPostUpdate')
        ) {
            return false;
        }

        if (
            version_compare(_PS_VERSION_, '1.5', '>=') &&
            (!$this->registerHook('displayHeader') ||
                !$this->registerHook('displayTop') ||
                !$this->registerHook('displayFooter') ||
                !$this->registerHook('displayProductButtons') ||
                !$this->registerHook('displayProductPriceBlock') ||
                !$this->registerHook('displayShoppingCartFooter'))
        ) {
            return false;
        }

        if (
            version_compare(_PS_VERSION_, '1.7', '>=') &&
            (!$this->registerHook('displayBeforeBodyClosingTag') ||
                !$this->registerHook('displayExpressCheckout') ||
                !$this->registerHook('paymentOptions'))
        ) {
            return false;
        }

        return true;
    }

    /**
     * Module uninstallation.
     *
     * @access public
     * @return bool Success of the uninstallation
     */
    public function uninstall()
    {
        if (!parent::uninstall()) {
            return false;
        }

        return true;
    }

    /**
     * Module Hook on Order status change
     *
     * @access public
     */
    public function hookActionOrderStatusPostUpdate($params)
    {
        \Classes\Helper\Logger::debug("hookActionOrderStatusPostUpdate: " . json_encode($params));
        $post = Tools::getAllValues();
        if (isset($post["cancel_product"]) || isset($post["partialRefundProduct"])) {
            return true;
        }
        \Classes\Helper\Logger::debug("hookActionOrderStatusPostUpdate post: " . json_encode($post));
        $capture_state_id = (int)\Configuration::get('PS_OS_PAYMENT');
        $refund_state_id = (int)\Configuration::get('PS_OS_REFUND');
        $cancel_state_id = (int)\Configuration::get('PS_OS_CANCELED');
        $current_state_id = (int)$params['newOrderStatus']->id;

        $order_id = (int)$params['id_order'];
        $order = new \Order($order_id);
        \Classes\Helper\Logger::debug("hookActionOrderStatusPostUpdate order: " . json_encode($order));
        if ((int)$order->current_state == (int)\Configuration::get('PS_OS_OUTOFSTOCK_UNPAID')) {
            $capture_state_id = (int)\Configuration::get('PS_OS_OUTOFSTOCK_PAID');
        }

        if ($order->payment == $this->name) {
            if ($current_state_id === $capture_state_id) {
                // Check to see if the order can be charged. Is there a charge Id for that order, and has it been authorised?
                if ($charge_id = $this->getChargeIdFromOrder($order)) {
                    if ($charge = \Classes\Helper\Api::getApi()->retrieveCharge($charge_id)) {
                        $isCapture = \Configuration::get('ZM_CAPTURE_METHOD') == 'capture' ? true : false;
                        \Classes\Helper\Logger::debug("Charge Retrieved - Apply Payment: " . json_encode(json_decode($charge->__toString())));
                        if ($charge->getState() == 'authorised' || ($charge->getState() == 'approved' && $isCapture === false)) {
                            \Classes\Helper\Logger::debug("state is authorised");
                            $response = \Classes\Helper\Api::getApi()->handleCapture($order, $charge);
                            return $response;
                        }
                    }
                }
            } elseif ($current_state_id === $refund_state_id) {
                // Is there a charge Id and has it been captured? If so, then refund it.
                $response = $this->_fullRefund($order);
                return $response;
            } elseif ($current_state_id === $cancel_state_id) {
                // Cancel the charge if it has been authorised and checkout.
                if ($charge_id = $this->getChargeIdFromOrder($order)) {
                    if ($charge = \Classes\Helper\Api::getApi()->retrieveCharge($charge_id)) {
                        \Classes\Helper\Logger::debug("Charge Retrieved - Canceled: " . json_encode(json_decode($charge->__toString())));
                        if ($charge->getState() == 'authorised') {
                            $response = \Classes\Helper\Api::getApi()->handleCancel($order, $charge);
                            return $response;
                        }
                    }
                }
            }

            \Classes\Helper\Logger::debug("hookActionOrderStatusPostUpdate: " . json_encode($params));
        }
    }

    /**
     * Return the Zip Charge Id from a Prestashop Order.
     *
     * @param  $order
     * @return String $charge_id
     */
    private function getChargeIdFromOrder($order)
    {
        $payments = $order->getOrderPaymentCollection();
        $charge_id = null;
        foreach ($payments as $key => $payment) {
            $charge_id = $payment->transaction_id;
        }
        return $charge_id;
    }

    /**
     * Module Hook Display Product Widget With Price
     */
    public function hookDisplayProductPriceBlock($params)
    {
        $enableProductWidget = Configuration::get('ZM_DISPLAY_WIDGET_PRODUCTPAGE');
        if (!$enableProductWidget) {
            return '';
        }
        $type = isset($params['type']) ? $params['type'] : '';
        $product = isset($params['product']) ? $params['product'] : null;
        $price = isset($product->price_amount) ? round($product->price_amount, 2) : 0;
        $productWidgetSelector = empty(trim(Configuration::get('ZM_PRODUCT_WIDGET_SELECTOR')))
            ? '' : trim(Configuration::get('ZM_PRODUCT_WIDGET_SELECTOR'));
        if ($type === 'after_price' && !$productWidgetSelector) {
            $tmpl = '<div data-zm-widget="popup" data-zm-asset="productwidget" data-zm-price="' . $price . '" data-zm-popup-asset="termsdialog"></div>';
            $tmpl .= '<script>if(typeof Zip !== "undefined" && Zip.Widget){Zip.Widget.render();}</script>';
            return $tmpl;
        } elseif ($type === 'after_price') {
            return '<script>var zip_widget_product_price="' . $price . '";if(typeof Zip !== "undefined" && Zip.Widget){Zip.Widget.render();}</script>';
        }
        return '';
    }

    /**
     * Prestashop 1.6 Cart Widget Display
     */
    public function hookDisplayShoppingCartFooter()
    {
        if (version_compare(_PS_VERSION_, '1.7', '>=')) {
            return '';
        } else {
            return $this->hookDisplayExpressCheckout();
        }
    }

    /**
     * Module Hook Display Cart Widget With Total
     */
    public function hookDisplayExpressCheckout()
    {
        $enableCartWidget = Configuration::get('ZM_DISPLAY_WIDGET_CARTPAGE');
        if (!$enableCartWidget) {
            return '';
        }
        if (isset($this->context->cart)) {
            $cart = $this->context->cart;
            $includeTax = true;
            $total = $cart->getOrderTotal($includeTax);

            $cartWidgetSelector = empty(trim(Configuration::get('ZM_CART_WIDGET_SELECTOR')))
                ? '' : trim(Configuration::get('ZM_CART_WIDGET_SELECTOR'));
            if ($cartWidgetSelector) {
                return '<script>var zip_widget_cart_price="' . $total . '";if(typeof Zip !== "undefined" && Zip.Widget){Zip.Widget.render();}</script>';
            } else {
                $tmpl = '<div data-zm-widget="popup" data-zm-asset="cartwidget" data-zm-price="' . $total . '" data-zm-popup-asset="termsdialog"></div>';
                $tmpl .= '<script>if(typeof Zip !== "undefined" && Zip.Widget){Zip.Widget.render();}</script>';
                return $tmpl;
            }
        }
        return '';
    }

    /**
     * Module Hook Display Top.
     *
     * @access public
     * @return  widget html
     */
    public function hookDisplayTop()
    {
        if (
            $this->context->controller->php_self == 'index' &&
            Configuration::get('ZM_DISPLAY_BANNER_HOMEPAGE')
        ) {
            return $this->zipWidgets->renderBanner();
        } elseif (
            $this->context->controller->php_self == 'product' &&
            Configuration::get('ZM_DISPLAY_BANNER_PRODUCTPAGE')
        ) {
            return $this->zipWidgets->renderBanner();
        } elseif (($this->context->controller->php_self == 'cart' || $this->context->controller->php_self == 'order') &&
            Configuration::get('ZM_DISPLAY_BANNER_CARTPAGE')
        ) {
            return $this->zipWidgets->renderBanner();
        } elseif (
            $this->context->controller->php_self == 'category' &&
            Configuration::get('ZM_DISPLAY_BANNER_CATEGORYPAGE')
        ) {
            return $this->zipWidgets->renderBanner();
        }
    }

    /**
     * Module Hook Payment Return.
     *
     * @access public
     * @return  widget html
     */
    public function hookPaymentReturn()
    {
        if (!$this->active) {
            return null;
        }

        return $this->fetchTemplate('views/templates/hooks/confirmation.tpl');
    }

    /**
     * Module Hook Payment Option for >= v1.7
     *
     * @access public
     * @param array $param
     * @return  array
     */
    public function hookPaymentOptions($params)
    {

        if (!$this->active || !$this->zm_is_active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        $minOrder = (float)(Tools::getValue('zm_minimum_order', Configuration::get('ZM_MINIMUM_ORDER')));
        if ($params['cart']->getOrderTotal() < $minOrder) {
            \Classes\Helper\Logger::info("Not loading zip payment method because Order amount is less that Zip Minimum Order.");
            return;
        }

        $checkout_title = Configuration::get('ZM_CHECKOUT_TEXT') ? Configuration::get('ZM_CHECKOUT_TEXT') : $this->l("Zip now, pay later");

        $this->context->smarty->assign('zm_logo_url', $this->getMethodIcon());

        $this->context->smarty->assign('zm_in_context', $this->useIframeCheckout() ? 'true' : 'false');

        $this->context->smarty->assign('checkout_title', $checkout_title);
        $this->context->smarty->assign('checkoutUri', $this->context->link->getModuleLink($this->name, 'payment', array(), true));
        $this->context->smarty->assign('redirectUri', $this->context->link->getModuleLink($this->name, 'validation', array(), true));

        $newOption = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $newOption->setCallToActionText($this->trans($checkout_title))
            ->setModuleName($this->name)
            ->setLogo($this->getMethodIcon())
            ->setAdditionalInformation($this->fetch('module:zipmoneypayment/views/templates/hooks/zipmoneypayment_intro.tpl'));

        $payment_options = array($newOption);

        return $payment_options;
    }

    /**
     * Module Hook Payment Option for < v1.6.
     *
     * @access public
     * @param array $param
     * @return  array
     */
    public function hookPayment($params)
    {
        if (!$this->active || !$this->zm_is_active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        $minOrder = (float)(Tools::getValue('zm_minimum_order', Configuration::get('ZM_MINIMUM_ORDER')));
        if ($params['cart']->getOrderTotal() < $minOrder) {
            return;
        }

        $checkout_title = Configuration::get('ZM_CHECKOUT_TEXT') ? Configuration::get('ZM_CHECKOUT_TEXT') : "Pay Later with zipPay";

        $this->context->smarty->assign('checkout_title', $checkout_title);
        $this->context->smarty->assign('zm_logo_url', $this->getMethodIcon());
        // $this->context->smarty->assign('payment_url', $this->context->link->getModuleLink($this->name, 'payment', array(), true));
        $this->context->smarty->assign('zm_in_context', $this->useIframeCheckout() ? 'true' : 'false');
        $this->context->smarty->assign('checkoutUri', $this->context->link->getModuleLink($this->name, 'payment', array(), true));
        $this->context->smarty->assign('redirectUri', $this->context->link->getModuleLink($this->name, 'validation', array(), true));

        return $this->fetchTemplate('views/templates/hooks/payment.tpl');
    }

    /**
     * Module Hook Admin Order.
     *
     * @access public
     * @param array $param
     * @return  widget html
     */
    public function hookAdminOrder($params)
    {
        $order = null;
        if (!empty($params['id_order']) && is_numeric($params['id_order'])) {
            $order = new Order($params['id_order']);
        }

        if ($order) {
            if ($order->payment == $this->name) {
                if (Tools::isSubmit('submitZipMoneyRefund')) {
                    $order->setCurrentState((int)\Configuration::get('PS_OS_REFUND'));
                    Tools::redirect($_SERVER['HTTP_REFERER']);
                }
                if (Tools::isSubmit('submitZipMoneyCancellation')) {
                    $order->setCurrentState((int)\Configuration::get('PS_OS_CANCELED'));
                    Tools::redirect($_SERVER['HTTP_REFERER']);
                }
                if (Tools::isSubmit('submitZipMoneyCapture')) {
                    $order_state = (int)\Configuration::get('PS_OS_PAYMENT');
                    if ((int)$order->current_state == (int)\Configuration::get('PS_OS_OUTOFSTOCK_UNPAID')) {
                        $order_state = (int)\Configuration::get('PS_OS_OUTOFSTOCK_PAID');
                    }

                    $history = new \OrderHistory();
                    $history->id_order = (int)$order->id;
                    $history->changeIdOrderState($order_state, (int)$order->id, true);
                    $history->addWithemail();

                    Tools::redirect($_SERVER['HTTP_REFERER']);
                }

                $order = new Order((int)$params['id_order']);
                $currency = new Currency($order->id_currency);

                if (version_compare(_PS_VERSION_, '1.5', '>=')) {
                    $order_state = (int)$order->current_state;
                } else {
                    $order_state = (int)OrderHistory::getLastOrderState($order->id);
                }

                $assign = array(
                    'base_url' => _PS_BASE_URL_ . __PS_BASE_URI__,
                    'module_name' => $this->name,
                    'order_state' => $order_state,
                    'params' => $params,
                    'id_currency' => $currency->getSign(),
                    'ps_version' => _PS_VERSION_,

                    'refund' => $this->_canRefund((int)$params['id_order']) && ((int)\Configuration::get('PS_OS_PAYMENT') === $order_state || (int)\Configuration::get('PS_OS_OUTOFSTOCK_PAID') === $order_state),
                    'capture' => \Classes\zipMoney::getOrderStatus('zip_authorised') == $order->current_state,
                    'cancel' => \Classes\zipMoney::getOrderStatus('zip_authorised') == $order->current_state
                );

                $this->context->smarty->assign($assign);
                $this->_html .= $this->fetchTemplate('/views/templates/admin/admin_order/actions.tpl');
                $this->_html .= '</fieldset>';
            }
        }

        return $this->_html;
    }

    /**
     * Module Hook Header.
     *
     * @access public
     */
    public function hookDisplayHeader()
    {
        $smarty = $this->context->smarty;

        $smarty->assign(array(
            'ZM_IN_CONTEXT' => Configuration::get('ZM_IN_CONTEXT')
        ));

        if (isset($this->context->cart) && $this->context->cart->id) {
            $this->context->smarty->assign('id_cart', (int)$this->context->cart->id);
        }
        if (self::isVersion17()) {
            $this->context->controller->registerStylesheet('modules-zipmoneypayment', 'modules/' . $this->name . '/views/css/zipmoney.css', array('media' => 'all', 'priority' => 150));
            $this->context->controller->registerJavascript('modules-zipmoneypayment', 'modules/' . $this->name . '/views/js/zipmoney.js', array('media' => 'all', 'priority' => 156));
        } else {
            $this->context->controller->addCss($this->_path . '/views/css/zipmoney.css');
            $this->context->controller->addJs($this->_path . '/views/js/zipmoney.js');
        }
    }

    /*
     * Handle Display Before Closing
     *
     * @access public
     * @return widget html
     */
    public function handleDisplayBeforeClosing()
    {
        $currencyCode = $this->context->currency->iso_code;
        $currencySymbol = $this->context->currency->symbol;
        $languageCode =  $this->context->language->iso_code;
        $region = 'au';
        if ($currencyCode) {
            switch ($currencyCode) {
                case 'GBP':
                    $region = 'gb';
                    break;
                case 'NZD':
                    $region = 'nz';
                    break;
                case 'USD':
                    $region = 'us';
                    break;
                case 'MXN':
                    $region = 'mx';
                    break;
                case 'SGD':
                    $region = 'sg';
                    break;
                case 'CAD':
                    $region = 'ca';
                    break;
                case 'AED':
                    $region = 'ae';
                    break;
                case 'ZAR':
                    $region = 'za';
                    break;
                case 'AUD':
                default:
                    $region = 'au';
                    break;
            }
        }
        $displayInline = Configuration::get('ZM_WIDGET_DISPLAY_INLINE') ? 'true' : 'false';
        $productWidgetSelector = empty(trim(Configuration::get('ZM_PRODUCT_WIDGET_SELECTOR')))
            ? '' : trim(Configuration::get('ZM_PRODUCT_WIDGET_SELECTOR'));
        $cartWidgetSelector = empty(trim(Configuration::get('ZM_CART_WIDGET_SELECTOR')))
            ? '' : trim(Configuration::get('ZM_CART_WIDGET_SELECTOR'));

        $process = '<script>
        var zip_widget_merchantId = "' . Configuration::get('ZM_MERCHANT_PUBLIC_KEY') . '";
        var zip_widget_env = "' . $this->getEnvironment() . '";
        var zip_widget_region = "' . $region . '";
        var zip_widget_language = "' . $languageCode . '";
        var zip_widget_currency_symbol = "' . $currencySymbol . '";
        var zip_widget_product_widget = "' . $productWidgetSelector . '";
        var zip_widget_cart_widget = "' . $cartWidgetSelector . '";
        var zip_widget_display_inline = ' . $displayInline . ';
        </script>' . "\n";
        $process .= '<script defer async src="https://static.zipmoney.com.au/lib/js/zm-widget-js/dist/zipmoney-widgets-v1.min.js"></script>';
        return $process;
    }

    /**
     * Handle Display only when necessary
     *
     * @access public
     * @return widget html
     */
    public function handleLibraryDisplay()
    {
        if (($this->context->controller->php_self == 'index' && Configuration::get('ZM_DISPLAY_BANNER_HOMEPAGE')) ||
            ($this->context->controller->php_self == 'product' && (Configuration::get('ZM_DISPLAY_BANNER_PRODUCTPAGE') ||
                Configuration::get('ZM_DISPLAY_WIDGET_PRODUCTPAGE'))) ||
            (($this->context->controller->php_self == 'cart' || $this->context->controller->php_self == 'order') && (Configuration::get('ZM_DISPLAY_BANNER_CARTPAGE') ||
                Configuration::get('ZM_DISPLAY_WIDGET_CARTPAGE'))) ||
            ($this->context->controller->php_self == 'category' && Configuration::get('ZM_DISPLAY_BANNER_CATEGORYPAGE')) ||
            ($this->context->controller->php_self == 'order')
        ) {
            return $this->handleDisplayBeforeClosing();
        }
        return;
    }

    /**
     * Module Hook Before Body Closing.
     *
     * @access public
     * @return  widget html
     */
    public function hookDisplayBeforeBodyClosingTag()
    {
        $process = $this->handleLibraryDisplay();
        return $process;
    }

    /**
     * Module Hook Before Body Closing.
     *
     * @access public
     * @return  widget html
     */
    public function hookDisplayFooter()
    {
        if (self::isVersion17()) {
            return false;
        }

        $process = $this->handleLibraryDisplay();
        return $process;
    }

    /**
     * Module Hook Add Order Slip. Create Partial Refund
     *
     * @access public
     * @param array $param
     */
    public function hookActionOrderSlipAdd($param)
    {
        \Classes\Helper\Logger::debug("hookActionOrderSlipAdd: " . json_encode($param));
        $post = Tools::getAllValues();
        \Classes\Helper\Logger::debug("hookActionOrderSlipAdd post: " . json_encode($post));
        $order = $param['order'];
        if ($order->payment == $this->name) {
            // $post = Tools::getAllValues();
            $shippingRefundAmount = $post["cancel_product"]["shipping_amount"];
            $productList = $param['productList'];
            $totalAmt = 0;
            $totalAmt += $shippingRefundAmount;
            if ($productList) {
                foreach ($productList as $key => $product) {
                    $totalAmt += $product['amount'];
                }
            }
            if ($totalAmt > 0) {
                if ($this->_canRefund($order)) {
                    $refundResponse = $this->_refund($order, round($totalAmt, 2), "Partial Refund");
                    if ($charge_id = $refundResponse["charge_id"]) {
                        $charge = \Classes\Helper\Api::getApi()->retrieveCharge($charge_id);
                        $refundedAmount = $charge["refunded_amount"];
                        if ($refundedAmount == $order->total_paid) {
                            /*** calling update method ***/
                            $refundState = (int)\Configuration::get('PS_OS_REFUND');
                            //  change status directly in Order
                            $order = new Order($order->id);
                            $order->setCurrentState($refundState);

                            // change status in OrderHistory
                            $history = new OrderHistory();
                            $history->changeIdOrderState($refundState, $order->id, true);
                        }
                    }
                }
            }
        }
    }

    /**
     * Module render form.
     *
     * @access public
     * @return  widget html
     */
    public function getContent()
    {
        $this->_postProcess();

        return $this->renderForm();
    }


    /**
     * Module check cart currency.
     *
     * @access public
     * @param object $cart
     * @return  bool
     */
    public function checkCurrency($cart)
    {
        $currency_order = new \Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Full Refund
     *
     * @access public
     * @param int $id_order , boolean $overrideOrderSlip (optional) [Used to control OrderSlip Module call]
     * @return  mixed (false)
     */
    private function _fullRefund($order, $overrideOrderSlip = false)
    {

        if (!$order) {
            return false;
        }

        if (!$this->_canRefund($order)) {
            return false;
        }

        if (!Validate::isLoadedObject($order)) {
            return false;
        }
        $products = $order->getProducts();
        \Classes\Helper\Logger::debug("_fullRefund products: " . json_encode($products));
        $currency = new Currency((int)$order->id_currency);

        if (!Validate::isLoadedObject($currency)) {
            $this->_errors[] = $this->l('Not a valid currency');
        }

        if (count($this->_errors)) {
            return false;
        }

        // Amount for refund
        $amt = 0.00;
        $productRefundTotal = 0.00;
        foreach ($products as $product) {
            $productRefundTotal += (float)($product['product_price_wt']) * ($product['product_quantity'] - $product['product_quantity_refunded']);
        }

        $amt = $productRefundTotal + (float)($order->total_shipping) + (float)($order->total_wrapping) - (float)($order->total_discounts);
        \Classes\Helper\Logger::debug("_fullRefund product refund amount: " . (string)$amt);
        // Create order slip for the whole order...
        if (version_compare(_PS_VERSION_, '1.6.0.10', '>=')) {
            $order_detail_list = array();
            if ($productRefundTotal > 0) {
                foreach ($products as $key => $product) {
                    $order_detail = new \OrderDetail($product['id_order_detail']);
                    if ($order_detail->product_quantity > $order_detail->product_quantity_refunded) {
                        \Classes\Helper\Logger::debug("_fullRefund order_details: " . json_encode($order_detail));
                        $order_detail_list[$product['id_order_detail']]['id_order_detail'] = $order_detail->id_order_detail;
                        $order_detail_list[$product['id_order_detail']]['quantity'] = $order_detail->product_quantity;
                        $order_detail_list[$product['id_order_detail']]['unit_price'] = $order_detail->unit_price_tax_incl;
                    }
                }
            }
            $shipping_cost_amount = $order->total_shipping;
            if (!$overrideOrderSlip) {
                \OrderSlip::create($order, $order_detail_list, $shipping_cost_amount, 0, false, false);
            }
        } else {
            $orderDetails = $order->getOrderDetailList();
            $productList = array();
            $qtyList = array();
            if ($productRefundTotal > 0) {
                foreach ($orderDetails as $detail) {
                    $productList[] = $detail['id_order_detail'];
                    $qtyList[] = $detail['product_quantity'];
                }
            }
            $shipping_cost = true;
            if (!$overrideOrderSlip) {
                \OrderSlip::createOrderSlip($order, $productList, $qtyList, $shipping_cost);
            }
        }

        // check if the refund is total or partial
        if (round((float)$order->total_paid_real, 2) == round((float)$amt, 2)) {
            $response = $this->_refund($order); // Full Refund
        } else {
            $response = $this->_refund($order, (float)($amt)); //Partial Refund
        }

        return $response;
    }


    /**
     * Handle Refund
     *
     * @access public
     * @param object $order , int $amt, string $reason
     * @return bool
     */
    private function _refund($order, $amt = null, $reason = null)
    {
        if (!$order) {
            die(Tools::displayError('Fatal Error: id_order is null'));
        }

        if (!$amt) {
            $amt = $order->total_paid;
        }

        if (!$reason) {
            $reason = "Full Refund";
        }

        $response = \Classes\Helper\Api::getApi()->handleRefund($order, $amt, $reason);
        if ($response) {
            if (array_key_exists('zm_error', $response)) {
                die(Tools::displayError('zipMoney Error: ' . $response['zm_error']));
            } elseif ($response == 'error') {
                die(Tools::displayError('zipMoney Error'));
            }
        }

        return $response;
    }

    /**
     * Check if the order is refundable
     *
     * @access private
     * @param int $id_order
     * @return bool
     */
    private function _canRefund($order)
    {
        if (!$order) {
            return false;
        }
        if (is_numeric($order)) {
            $order = new \Order($order);
        }
        if ($order->payment != $this->name) {
            return false;
        }

        return true;
    }

    /**
     * Render Form
     *
     * @access public
     * @return  mixed (false)
     */
    public function renderForm()
    {
        $pre16 = version_compare(_PS_VERSION_, '1.6', '<');

        $switch_type = $pre16 ? 'radio' : 'switch';
        $minimumAmountField = $pre16 ?
            array(
                'type' => 'text',
                'label' => $this->l("Minimum Order Value"),
                'desc' => $this->l('(Must be number) You can set the minimum order/cart value for Zip to show at checkout.'),
                'name' => 'zm_minimum_order',
                'placeholder' => '0'
            ) :
            array(
                'type' => 'html',
                'label' => $this->l("Minimum Order Value"),
                'desc' => $this->l('You can set the minimum order/cart value for Zip to show at checkout.'),
                'name' => 'zm_minimum_order',
                'size' => 32,
                'required' => true,
                'html_content' => "<input type='number' name='zm_minimum_order' id='zm_minimum_order' required='required' value='" . (float)Tools::getValue('zm_minimum_order', Configuration::get('ZM_MINIMUM_ORDER')) . "' class='form-control' />"
            );

        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Merchant Credentials', array(), 'Modules.Zipmoneypayment.Zipmoneypayment'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'type' => $switch_type,
                        'label' => $this->l('Active'),
                        'name' => "zm_is_active",
                        'is_bool' => true,
                        'class' => 't',
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled'),
                            )
                        ),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->trans('Public Key', array(), 'Modules.Zipmoneypayment.Zipmoneypayment'),
                        'desc' => $this->l('This public key only use in Australia to display the zip widget in product and cart page.'),
                        'name' => 'zm_merchant_public_key',
                        'required' => false,
                        'placeholder' => 'xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx'
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->trans('Private Key', array(), 'Modules.Zipmoneypayment.Zipmoneypayment'),
                        'desc' => $this->l('Please contact integrations@zip.co to request Private key.'),
                        'name' => 'zm_merchant_private_key',
                        'size' => 32,
                        'required' => true,
                        'placeholder' => '*******************************'
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Environment'),
                        'name' => "zm_environment",
                        'class' => 't',
                        'options' => array(
                            'query' => array(
                                array(
                                    'id' => 'sandbox',
                                    'name' => $this->l('Sandbox'),
                                ),
                                array(
                                    'id' => 'production',
                                    'name' => $this->l('Production'),
                                )
                            ),
                            'id' => 'id', 'name' => 'name'
                        )
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Capture Method'),
                        'desc' => $this->l('You can set whether you would like to authorise a payment first, then capture it manually or to capture the payment as soon as a checkout is complete.'),
                        'name' => "zm_capture_method",
                        'is_bool' => true,
                        'class' => 't',
                        'options' => array(
                            'query' => array(
                                array(
                                    'id' => 'authorise',
                                    'value' => true,
                                    'name' => $this->l('Authorise'),
                                ),
                                array(
                                    'id' => 'capture',
                                    'value' => false,
                                    'name' => $this->l('Capture'),
                                )
                            ),
                            'id' => 'id', 'name' => 'name'

                        )
                    ),
                    array(
                        'type' => $switch_type,
                        'label' => $this->l('In-Context Checkout'),
                        'desc' => $this->l('In-Context (Iframe) Checkout may not work on some browsers. This feature only work for AU region. Other region it will redirect to zip for payment.'),
                        'name' => "zm_is_incontext",
                        'is_bool' => true,
                        'class' => 't',
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled'),
                            )
                        ),
                    ),
                    $minimumAmountField,
                    array(
                        'type' => 'select',
                        'label' => $this->l('Log Setting'),
                        'name' => "zm_logging",
                        'required' => true,
                        'class' => 't',
                        'options' => array(
                            'query' => array(
                                array(
                                    'id' => \Classes\Helper\Logger::DEBUG,
                                    'name' => $this->l('Debug')
                                ),
                                array(
                                    'id' => \Classes\Helper\Logger::INFO,
                                    'name' => $this->l('Info')
                                ),
                                array(
                                    'id' => 0,
                                    'name' => $this->l('None')
                                )
                            ),
                            'id' => 'id', 'name' => 'name'
                        )
                    )
                ),
            ),
        );

        $fields_form_customization = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Widgets', array(), 'Modules.Zipmoneypayment.Zipmoneypayment'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'type' => $switch_type,
                        'label' => $this->l('Display Widget inline or iframe'),
                        'name' => "zm_widget_display_inline",
                        'is_bool' => true,
                        'class' => 't',
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Inline'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Iframe'),
                            )
                        ),
                    ),
                    array(
                        'type' => $switch_type,
                        'label' => $this->l('Display Widget in Product Page'),
                        'name' => "zm_display_widget_productpage",
                        'is_bool' => true,
                        'class' => 't',
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled'),
                            )
                        ),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Product Widget Selector'),
                        'name' => 'zm_product_widget_selector',
                        'size' => 200,
                        'desc' => $this->l('CSS selector where you want to display the product widget, for example: ".product-container .product-prices" leave it empty will fallback to default'),
                    ),
                    array(
                        'type' => $switch_type,
                        'label' => $this->l('Display Widget in Cart Page'),
                        'name' => "zm_display_widget_cartpage",
                        'is_bool' => true,
                        'class' => 't',
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled'),
                            )
                        ),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Cart Widget Selector'),
                        'name' => 'zm_cart_widget_selector',
                        'size' => 200,
                        'desc' => $this->l('CSS selector where you want to display the cart widget, for example: ".cart-grid .cart-summar" leave it empty will fallback to default'),
                    ),
                    array(
                        'type' => $switch_type,
                        'label' => $this->l('Display Banner in Home Page'),
                        'name' => "zm_display_banner_homepage",
                        'is_bool' => true,
                        'class' => 't',
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled'),
                            )
                        ),
                    ),
                    array(
                        'type' => $switch_type,
                        'label' => $this->l('Display Banner in Category Page'),
                        'name' => "zm_display_banner_categorypage",
                        'is_bool' => true,
                        'class' => 't',
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled'),
                            )
                        ),
                    ),
                    array(
                        'type' => $switch_type,
                        'label' => $this->l('Display Banner in Product Page'),
                        'name' => "zm_display_banner_productpage",
                        'is_bool' => true,
                        'class' => 't',
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled'),
                            )
                        ),
                    ),
                    array(
                        'type' => $switch_type,
                        'label' => $this->l('Display Banner in Cart Page'),
                        'name' => "zm_display_banner_cartpage",
                        'is_bool' => true,
                        'class' => 't',
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled'),
                            )
                        ),
                    )
                ),
                'submit' => array(
                    'title' => $this->trans('Save', array(), 'Modules.Zipmoneypayment.Zipmoneypayment')
                )
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ?: 0;
        $this->fields_form = array();
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues()
        );

        return $helper->generateForm(array($fields_form, $fields_form_customization));
    }

    /**
     * Returns config values
     *
     * @access public
     * @return  array
     */
    public function getConfigFieldsValues()
    {
        return array(
            'zm_is_active' => Tools::getValue('zm_is_active', Configuration::get('ZM_IS_ACTIVE')),
            'zm_environment' => Tools::getValue('zm_environment', Configuration::get('ZM_ENVIRONMENT')),
            'zm_capture_method' => Tools::getValue('zm_capture_method', Configuration::get('ZM_CAPTURE_METHOD')),
            'zm_merchant_public_key' => Tools::getValue('zm_merchant_public_key', Configuration::get('ZM_MERCHANT_PUBLIC_KEY')),
            'zm_merchant_private_key' => Tools::getValue('zm_merchant_private_key', Configuration::get('ZM_MERCHANT_PRIVATE_KEY')),
            'zm_is_incontext' => (int)Tools::getValue('zm_is_incontext', Configuration::get('ZM_IN_CONTEXT')),
            'zm_minimum_order' => (float)(Tools::getValue('zm_minimum_order', Configuration::get('ZM_MINIMUM_ORDER'))),
            'zm_logging' => (int)Tools::getValue('zm_logging', Configuration::get('ZM_LOGGING')),
            'zm_checkout_text' => Tools::getValue('zm_checkout_text', Configuration::get('ZM_CHECKOUT_TEXT')),
            'zm_widget_display_inline' => Tools::getValue('zm_widget_display_inline', Configuration::get('ZM_WIDGET_DISPLAY_INLINE')),
            'zm_display_widget_productpage' => Tools::getValue('zm_display_widget_productpage', Configuration::get('ZM_DISPLAY_WIDGET_PRODUCTPAGE')),
            'zm_product_widget_selector' => Tools::getValue('zm_product_widget_selector', Configuration::get('ZM_PRODUCT_WIDGET_SELECTOR')),
            'zm_display_widget_cartpage' => Tools::getValue('zm_display_widget_cartpage', Configuration::get('ZM_DISPLAY_WIDGET_CARTPAGE')),
            'zm_cart_widget_selector' => Tools::getValue('zm_cart_widget_selector', Configuration::get('ZM_CART_WIDGET_SELECTOR')),
            'zm_display_banner_homepage' => Tools::getValue('zm_display_banner_homepage', Configuration::get('ZM_DISPLAY_BANNER_HOMEPAGE')),
            'zm_display_banner_categorypage' => Tools::getValue('zm_display_banner_categorypage', Configuration::get('ZM_DISPLAY_BANNER_CATEGORYPAGE')),
            'zm_display_banner_cartpage' => Tools::getValue('zm_display_banner_cartpage', Configuration::get('ZM_DISPLAY_BANNER_CARTPAGE')),
            'zm_display_banner_productpage' => Tools::getValue('zm_display_banner_productpage', Configuration::get('ZM_DISPLAY_BANNER_PRODUCTPAGE'))
        );
    }

    /**
     * Returns config values
     *
     * @access private
     * @return array
     */
    private function _postProcess()
    {
        if (Tools::isSubmit('btnSubmit')) {
            if (!Tools::getValue('zm_merchant_public_key')) {
                $post_errors[] = $this->l('ZipPay Merchant ID cannot be empty');
            }

            if (!Tools::getValue('zm_merchant_private_key')) {
                $post_errors[] = $this->l('ZipPay Merchant Key cannot be empty');
            }

            if (empty($post_errors)) {
                Configuration::updateValue('ZM_IS_ACTIVE', (int)Tools::getValue('zm_is_active'));
                Configuration::updateValue('ZM_ENVIRONMENT', Tools::getValue('zm_environment'));
                Configuration::updateValue('ZM_CAPTURE_METHOD', Tools::getValue('zm_capture_method'));
                Configuration::updateValue('ZM_MERCHANT_PUBLIC_KEY', trim(Tools::getValue('zm_merchant_public_key')));
                Configuration::updateValue('ZM_MERCHANT_PRIVATE_KEY', trim(Tools::getValue('zm_merchant_private_key')));
                Configuration::updateValue('ZM_IN_CONTEXT', trim(Tools::getValue('zm_is_incontext')));
                Configuration::updateValue('ZM_MINIMUM_ORDER', (float)(Tools::getValue('zm_minimum_order')));
                Configuration::updateValue('ZM_LOGGING', (int)Tools::getValue('zm_logging'));
                Configuration::updateValue('ZM_CHECKOUT_TEXT', trim(Tools::getValue('zm_checkout_text')));

                Configuration::updateValue('ZM_WIDGET_DISPLAY_INLINE', (int)Tools::getValue('zm_widget_display_inline'));
                Configuration::updateValue('ZM_DISPLAY_WIDGET_PRODUCTPAGE', (int)Tools::getValue('zm_display_widget_productpage'));
                Configuration::updateValue('ZM_PRODUCT_WIDGET_SELECTOR', Tools::getValue('zm_product_widget_selector'));
                Configuration::updateValue('ZM_DISPLAY_WIDGET_CARTPAGE', (int)Tools::getValue('zm_display_widget_cartpage'));
                Configuration::updateValue('ZM_CART_WIDGET_SELECTOR', Tools::getValue('zm_cart_widget_selector'));

                Configuration::updateValue('ZM_DISPLAY_BANNER_HOMEPAGE', (int)Tools::getValue('zm_display_banner_homepage'));
                Configuration::updateValue('ZM_DISPLAY_BANNER_CATEGORYPAGE', (int)Tools::getValue('zm_display_banner_categorypage'));
                Configuration::updateValue('ZM_DISPLAY_BANNER_PRODUCTPAGE', (int)Tools::getValue('zm_display_banner_productpage'));
                Configuration::updateValue('ZM_DISPLAY_BANNER_CARTPAGE', (int)Tools::getValue('zm_display_banner_cartpage'));

                \zipMoney\Configuration::getDefaultConfiguration()->setApiKey('Authorization', "Bearer " . \Configuration::get('ZM_MERCHANT_PRIVATE_KEY'));
                \zipMoney\Configuration::getDefaultConfiguration()->setEnvironment($this->getEnvironment());

                if ($this->_refreshApiKeyHash()) {
                    \Classes\Helper\Logger::info("Handling config update");
                }

                $this->_html .= $this->displayConfirmation($this->trans('Settings updated', array(), 'Modules.Zipmoneypayment.Zipmoneypayment'));
            } else {
                $this->context->smarty->assign('ZipPay_save_fail', true);
                $this->context->smarty->assign('ZipPay_errors', $post_errors);
            }
        }
    }

    /**
     * Refreshes Api Key Hash
     *
     * @access private
     * @return  bool
     */
    private function _refreshApiKeyHash()
    {
        $current_hash = Configuration::get('ZM_API_KEY_HASH');
        $merchant_id = Configuration::get('ZM_MERCHANT_PUBLIC_KEY');
        $merchant_key = Configuration::get('ZM_MERCHANT_PRIVATE_KEY');

        $new_hash = md5(serialize(array($merchant_id, $merchant_key)));

        if ($current_hash != $new_hash) {
            Configuration::updateValue('ZM_API_KEY_HASH', $new_hash);
            return true;
        }
        return false;
    }


    /**
     * Create Order State
     *
     * @access private
     */
    private function _createOrderState()
    {

        $order_states = array(
            array(
                "name" => "zipMoney Pending",
                "send_email" => false,
                "color" => "#4169E1",
                "hidden" => false,
                "module_name" => $this->name,
                "template" => "zip_pending",
                "delivery" => false,
                "logable" => false,
                "invoice" => false
            ),

            array(
                "name" => "zipMoney Authorised",
                "send_email" => false,
                "color" => "#00bb44",
                "hidden" => false,
                "module_name" => $this->name,
                "template" => "zip_authorised",
                "delivery" => false,
                "logable" => false,
                "invoice" => false
            ),

            array(
                "name" => "zipMoney Under Review",
                "send_email" => false,
                "color" => "#975d00",
                "hidden" => false,
                "module_name" => $this->name,
                "template" => "zip_under_review",
                "delivery" => false,
                "logable" => false,
                "invoice" => false
            )
        );

        foreach ($order_states as $oState) {
            $order_state = \Classes\zipMoney::getOrderStatus($oState['template']);

            if (!$order_state) {
                // Create Pending Status
                $order_state = new OrderState();
                $order_state->name = array();

                foreach (Language::getLanguages() as $language) {
                    $order_state->name[$language['id_lang']] = $oState['name'];
                }

                $order_state->send_email = $oState['send_email'];
                $order_state->color = $oState['color'];
                $order_state->hidden = $oState['hidden'];
                $order_state->module_name = $oState['module_name'];
                $order_state->template = $oState['template'];
                $order_state->delivery = $oState['delivery'];
                $order_state->logable = $oState['logable'];
                $order_state->invoice = $oState['invoice'];

                $order_state->add();
            }
        }
    }


    /**
     * Check if incontext checkout has been enabled
     *
     * @access public
     * @return bool
     */
    public function useIframeCheckout()
    {
        $currentCurrency = $this->context->currency;
        $currencyCode = $currentCurrency->iso_code;
        if ($currencyCode == CurrencyUtil::CURRENCY_AUD) {
            return Configuration::get('ZM_IN_CONTEXT');
        }
        return false; // Disable iFrame checkout until we fix the checkout js issue that supports iFrame
    }

    /**
     * Fetches the specified template
     *
     * @access public
     * @param $namecheck
     * @return html
     */
    public function fetchTemplate($name)
    {
        if (version_compare(_PS_VERSION_, '1.4', '<')) {
            $this->context->smarty->currentTemplate = $name;
        } elseif (version_compare(_PS_VERSION_, '1.5', '<')) {
            $views = 'views/templates/';
            if (@filemtime(dirname(__FILE__) . '/' . $name)) {
                return $this->display(__FILE__, $name);
            } elseif (@filemtime(dirname(__FILE__) . '/' . $views . 'hook/' . $name)) {
                return $this->display(__FILE__, $views . 'hook/' . $name);
            } elseif (@filemtime(dirname(__FILE__) . '/' . $views . 'front/' . $name)) {
                return $this->display(__FILE__, $views . 'front/' . $name);
            } elseif (@filemtime(dirname(__FILE__) . '/' . $views . 'admin/' . $name)) {
                return $this->display(__FILE__, $views . 'admin/' . $name);
            }
        }

        return $this->display(__FILE__, $name);
    }

    /**
     * Returns the method icon
     *
     * @access private
     * @return string
     */
    public function getMethodIcon()
    {
        return $this->_logo_src_url;
    }

    /**
     * Returns the api environment
     *
     * @access public
     * @return string
     */
    public function getEnvironment()
    {
        return \Configuration::get('ZM_ENVIRONMENT');
    }

    /**
     * Return the merchant private key
     *
     * @access public
     * @return String
     */
    public function getMerchantKey()
    {
        return \Configuration::get('ZM_MERCHANT_PRIVATE_KEY');
    }

    /**
     * Module for translating String with version compatibility.
     *
     * @param int $id
     * @param array $parameters
     * @param  $domain
     * @param  $locale
     * @access protected
     * @return String
     */
    protected function trans($id, array $parameters = array(), $domain = null, $locale = null)
    {
        if (version_compare(_PS_VERSION_, '1.7', '>=')) {
            return parent::trans($id, $parameters, $domain, $locale);
        } else {
            return $this->l($id);
        }
    }

    /**
     * Determines if prestashop version is 1.7.*
     *
     * @return boolean
     */
    protected static function isVersion17()
    {
        return version_compare(_PS_VERSION_, '1.7', '>=');
    }
}
